@@ -149,6 +149,22 @@ public:
         return(rep);
     }
 
+    SEXP execv(std::vector<std::string> cmd) {
+        std::vector<const char*> cmdv(cmd.size());
+        std::vector<size_t> cmdlen(cmd.size());
+        for (unsigned int i=0; i < cmd.size(); ++i) {
+          cmdv[i] = cmd[i].c_str();
+          cmdlen[i] = cmd[i].size();
+        }
+
+        redisReply *reply = 
+            static_cast<redisReply*>(redisCommandArgv(prc_, cmd.size(), 
+                                                      &(cmdv[0]), &(cmdlen[0])));
+        SEXP rep = extract_reply(reply);
+        freeReplyObject(reply);
+        return(rep);
+    }
+
 
     // redis set -- serializes to R internal format
     std::string set(std::string key, SEXP s) {
@@ -179,6 +195,87 @@ public:
         return(obj);
     }
 
+    // redis set -- serializes to R internal format
+    int hset(std::string key, std::string field, SEXP s) {
+
+        // if raw, use as is else serialize to raw
+        Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : serializeToRaw(s);
+
+        // uses binary protocol, see hiredis doc at github
+        redisReply *reply =
+            static_cast<redisReply*>(redisCommand(prc_, "HSET %s %s %b",
+                                                  key.c_str(), field.c_str(), x.begin(), x.size()));
+
+        checkReplyType(reply, replyInteger_t); // ensure we got integer
+        int res = reply->integer;
+        freeReplyObject(reply);
+        return(res);
+    }
+
+    // redis get -- deserializes from R format
+    SEXP hget(std::string key, std::string field) {
+
+        redisReply *reply =
+            static_cast<redisReply*>(redisCommand(prc_, "HGET %s %s", key.c_str(), field.c_str()));
+
+        int nc = reply->len;
+        Rcpp::RawVector res(nc);
+        memcpy(res.begin(), reply->str, nc);
+        freeReplyObject(reply);
+        SEXP obj = unserializeFromRaw(res);
+        return(obj);
+    }
+
+    // redis sadd -- serializes to R internal format
+    SEXP sadd(std::string key, SEXP s) {
+
+        // if raw, use as is else serialize to raw
+        Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : serializeToRaw(s);
+        const char* cmdv[3] = {"SADD", key.c_str(), reinterpret_cast<char*>(x.begin())};
+        size_t cmdlen[3] = {4, key.length(), static_cast<size_t>(x.size())};
+
+        // uses binary protocol, see hiredis doc at github
+        redisReply *reply = static_cast<redisReply*>(redisCommandArgv(prc_, 3, cmdv, cmdlen));
+        SEXP rep = extract_reply(reply);
+        freeReplyObject(reply);
+        return(rep);
+    }
+
+    // redis srem -- serializes to R internal format
+    SEXP srem(std::string key, SEXP s) {
+
+        // if raw, use as is else serialize to raw
+        Rcpp::RawVector x = (TYPEOF(s) == RAWSXP) ? s : serializeToRaw(s);
+        const char* cmdv[3] = {"SREM", key.c_str(), reinterpret_cast<char*>(x.begin())};
+        size_t cmdlen[3] = {4, key.length(), static_cast<size_t>(x.size())};
+
+        // uses binary protocol, see hiredis doc at github
+        redisReply *reply = static_cast<redisReply*>(redisCommandArgv(prc_, 3, cmdv, cmdlen));
+        SEXP rep = extract_reply(reply);
+        freeReplyObject(reply);
+        return(rep);
+    }
+
+    // redis smembers -- deserializes from R format
+    Rcpp::List smembers(std::string key) {
+
+        redisReply *reply = 
+            static_cast<redisReply*>(redisCommand(prc_, "SMEMBERS %s", key.c_str()));
+
+        unsigned int len = reply->elements;
+        Rcpp::List x(len);
+        for (unsigned int i = 0; i < len; i++) {
+            int nc = reply->element[i]->len;
+            Rcpp::RawVector res(nc);
+            memcpy(res.begin(), reply->element[i]->str, nc);
+            SEXP obj = unserializeFromRaw(res);
+            x[i] = obj;
+        }
+
+        freeReplyObject(reply);
+        return(x);
+    }
+
     // redis keys -- returns character vector
     SEXP keys(std::string regexp) {
 
@@ -491,10 +588,18 @@ RCPP_MODULE(Redis) {
         .constructor<std::string, int>("constructor with host and port")  
 
         .method("exec", &Redis::exec,  "execute given redis command and arguments")
+        .method("execv", &Redis::execv,  "execute given a vector of redis command and arguments")
 
         .method("set",  &Redis::set,   "runs 'SET key object', serializes internally")
         .method("get",  &Redis::get,   "runs 'GET key', deserializes internally")
 
+        .method("hset",  &Redis::hset,   "runs 'HSET key field object', serializes internally")
+        .method("hget",  &Redis::hget,   "runs 'HGET key field', deserializes internally")
+
+        .method("sadd",     &Redis::sadd,     "runs 'SADD key member', serializes internally")
+        .method("srem",     &Redis::srem,     "runs 'SREM key member', serializes internally")
+        .method("smembers", &Redis::smembers, "runs 'SMEMBERS key', deserializes internally")
+
         .method("keys", &Redis::keys,  "runs 'KEYS expr', returns character vector")
 
         .method("lrange",  &Redis::lrange,   "runs 'LRANGE key start end' for list")
